@@ -14,6 +14,42 @@ test_succeeds("model can be saved and loaded", {
   model <- load_model_hdf5(tmp)
 })
 
+test_succeeds("model with custom loss and metrics can be saved and loaded", {
+  
+  if (!keras:::have_h5py())
+    skip("h5py not available for testing")
+  
+  model <- define_model()
+  
+  sparse_top_k_cat_acc <- function(y_pred, y_true){
+    metric_sparse_top_k_categorical_accuracy(y_pred, y_true, k = 5)
+  }
+  
+  custom_loss <- function(y_pred, y_true) {
+    loss_categorical_crossentropy(y_pred, y_true)
+  }
+  
+  model %>% compile(
+    loss = custom_loss,
+    optimizer = optimizer_nadam(),
+    metrics = c(top_k_acc = sparse_top_k_cat_acc)
+  )
+  
+  tmp <- tempfile("model", fileext = ".hdf5")
+  save_model_hdf5(model, tmp)
+  model <- load_model_hdf5(tmp, custom_objects = c(top_k_acc = sparse_top_k_cat_acc,
+                                                   custom_loss = custom_loss))
+  
+  # generate dummy training data
+  data <- matrix(rexp(1000*784), nrow = 1000, ncol = 784)
+  labels <- matrix(round(runif(1000*10, min = 0, max = 9)), nrow = 1000, ncol = 10)
+  
+  
+  model %>% fit(data, labels, epochs = 2)
+  
+})
+
+
 test_succeeds("model weights can be saved and loaded", {
 
   if (!keras:::have_h5py())
@@ -56,10 +92,6 @@ test_succeeds("model can be saved and loaded from R 'raw' object", {
 })
 
 test_succeeds("saved models/weights are mirrored in the run_dir", {
-  
-  if (packageVersion("tfruns") < "0.9.1.9002")
-    skip("required version of tfruns not available")
-  
   run <- tfruns::training_run("train.R")
   run_dir <- run$run_dir
   expect_true(file.exists(file.path(run_dir, "model.h5")))
@@ -67,10 +99,6 @@ test_succeeds("saved models/weights are mirrored in the run_dir", {
 })
 
 test_succeeds("callback output is redirected to run_dir", {
-  
-  if (packageVersion("tfruns") < "0.9.1.9002")
-    skip("required version of tfruns not available")
-  
   run <- tfruns::training_run("train.R")
   run_dir <- run$run_dir
   if (is_backend("tensorflow"))
